@@ -258,22 +258,26 @@ int main(int argc, char **argv)
     });
     QTextStream(stdout) << "QImageReader::supportedImageFormats: " << formatStrings.join(", ") << "\n";
 
+    // checks if the format has read capability
     if (!formats.contains(format)) {
-        if (format == "avci" || format == "heif" || format == "hej2") {
-            QTextStream(stdout) << "WARNING : " << suffix << " is not supported with current libheif configuration!\n"
-                                << "********* "
-                                << "Finished basic read tests for " << suffix << " images *********\n";
-            return 0;
-        }
+        QTextStream(stdout) << "FAIL : current configuration is missing necessary decoder(s) for " << suffix << "!\n"
+                            << "********* "
+                            << "Finished basic read tests for " << suffix << " images *********\n";
+        return 1;
     }
 
     const QFileInfoList lstImgDir = imgdir.entryInfoList();
-    // Launch 2 runs for each test: first run on a random access device, second run on a sequential access device
-    for (int seq = 0; seq < 2; ++seq) {
+    // Launch 3 runs for each test:
+    // - first run on a random access device with allocation limit set to 256 MiB.
+    // - second run on a random access device with allocation limit set to 0 MiB.
+    // - third run on a sequential access device.
+    for (int run = 0; run < 3; ++run) {
+        QImageReader::setAllocationLimit(run == 1 ? 0 : 256);
+        bool seq = run == 2;
         if (seq) {
             QTextStream(stdout) << "* Run on SEQUENTIAL ACCESS device\n";
         } else {
-            QTextStream(stdout) << "* Run on RANDOM ACCESS device\n";
+            QTextStream(stdout) << "* Run on RANDOM ACCESS device (allocation limit: " << QImageReader::allocationLimit() << " MiB)\n";
         }
         for (const QFileInfo &fi : lstImgDir) {
             TemplateImage timg(fi);
@@ -281,6 +285,12 @@ int main(int argc, char **argv)
                 continue;
             }
 
+            if (seq && timg.skipSequentialDeviceTest()) {
+                QTextStream(stdout) << "SKIP : " << fi.fileName() << ": marked to be skipped on a sequential device (don't worry, it's ok)\n";
+                ++skipped;
+                continue;
+            }
+
             TemplateImage::TestFlags flags = TemplateImage::None;
             QString comment;
             QFileInfo expFileInfo = timg.compareImage(flags, comment);
@@ -332,12 +342,7 @@ int main(int argc, char **argv)
             OptionTest optionTest;
             if (!optionTest.store(&inputReader)) {
                 QTextStream(stdout) << "FAIL : " << fi.fileName() << ": error while reading options\n";
-                if (format == "heif") {
-                    // libheif + ffmpeg decoder is unable to load all HEIF files.
-                    ++skipped;
-                } else {
-                    ++failed;
-                }
+                ++failed;
                 continue;
             }
 
