@@ -7,6 +7,10 @@
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
+#include "exr_p.h"
+#include "scanlineconverter_p.h"
+#include "util_p.h"
+
 /* *** EXR_CONVERT_TO_SRGB ***
  * If defined, the linear data is converted to sRGB on read to accommodate
  * programs that do not support color profiles.
@@ -28,7 +32,7 @@
  * The maximum size in pixel allowed by the plugin.
  */
 #ifndef EXR_MAX_IMAGE_WIDTH
-#define EXR_MAX_IMAGE_WIDTH 300000
+#define EXR_MAX_IMAGE_WIDTH KIF_LARGE_IMAGE_PIXEL_LIMIT
 #endif
 #ifndef EXR_MAX_IMAGE_HEIGHT
 #define EXR_MAX_IMAGE_HEIGHT EXR_MAX_IMAGE_WIDTH
@@ -50,10 +54,6 @@
 #define EXR_LINES_PER_BLOCK 128
 #endif
 
-#include "exr_p.h"
-#include "scanlineconverter_p.h"
-#include "util_p.h"
-
 #include <IexThrowErrnoExc.h>
 #include <ImathBox.h>
 #include <ImfArray.h>
@@ -75,14 +75,20 @@
 
 #include <QColorSpace>
 #include <QDataStream>
-#include <QDebug>
 #include <QFloat16>
 #include <QImage>
 #include <QImageIOPlugin>
 #include <QLocale>
+#include <QLoggingCategory>
 #include <QThread>
 #include <QTimeZone>
 
+#ifdef QT_DEBUG
+Q_LOGGING_CATEGORY(LOG_EXRPLUGIN, "kf.imageformats.plugins.exr", QtDebugMsg)
+#else
+Q_LOGGING_CATEGORY(LOG_EXRPLUGIN, "kf.imageformats.plugins.exr", QtWarningMsg)
+#endif
+
 class K_IStream : public Imf::IStream
 {
 public:
@@ -236,10 +242,10 @@ static QStringList viewList(const Imf::Header &h)
 }
 
 #ifdef QT_DEBUG
-void printAttributes(const Imf::Header &h)
+static void printAttributes(const Imf::Header &h)
 {
     for (auto i = h.begin(); i != h.end(); ++i) {
-        qDebug() << i.name();
+        qCDebug(LOG_EXRPLUGIN) << i.name();
     }
 }
 #endif
@@ -288,8 +294,14 @@ static void readMetadata(const Imf::Header &header, QImage &image)
         if (auto pixelAspectRatio = header.findTypedAttribute<Imf::FloatAttribute>("pixelAspectRatio")) {
             par = pixelAspectRatio->value();
         }
-        image.setDotsPerMeterX(qRound(xDensity->value() * 100.0 / 2.54));
-        image.setDotsPerMeterY(qRound(xDensity->value() * par * 100.0 / 2.54));
+        auto hres = dpi2ppm(xDensity->value());
+        if (hres > 0) {
+            image.setDotsPerMeterX(hres);
+        }
+        auto vres = dpi2ppm(xDensity->value() * par);
+        if (vres > 0) {
+            image.setDotsPerMeterY(vres);
+        }
     }
 
     // Non-standard attribute
@@ -379,19 +391,20 @@ bool EXRHandler::read(QImage *outImage)
 
         // limiting the maximum image size on a reasonable size (as done in other plugins)
         if (width > EXR_MAX_IMAGE_WIDTH || height > EXR_MAX_IMAGE_HEIGHT) {
-            qWarning() << "The maximum image size is limited to" << EXR_MAX_IMAGE_WIDTH << "x" << EXR_MAX_IMAGE_HEIGHT << "px";
+            qCWarning(LOG_EXRPLUGIN) << "The maximum image size is limited to" << EXR_MAX_IMAGE_WIDTH << "x" << EXR_MAX_IMAGE_HEIGHT << "px";
             return false;
         }
 
         // creating the image
         QImage image = imageAlloc(width, height, imageFormat(file));
         if (image.isNull()) {
-            qWarning() << "Failed to allocate image, invalid size?" << QSize(width, height);
+            qCWarning(LOG_EXRPLUGIN) << "Failed to allocate image, invalid size?" << QSize(width, height);
             return false;
         }
 
         Imf::Array2D<Imf::Rgba> pixels;
         pixels.resizeErase(EXR_LINES_PER_BLOCK, width);
+        std::memset(pixels[0], 0, sizeof(Imf::Rgba) * EXR_LINES_PER_BLOCK * width);
         bool isRgba = image.hasAlphaChannel();
 
         for (int y = 0, n = 0; y < height; y += n) {
@@ -554,7 +567,7 @@ bool EXRHandler::write(const QImage &image)
 
         // limiting the maximum image size on a reasonable size (as done in other plugins)
         if (width > EXR_MAX_IMAGE_WIDTH || height > EXR_MAX_IMAGE_HEIGHT) {
-            qWarning() << "The maximum image size is limited to" << EXR_MAX_IMAGE_WIDTH << "x" << EXR_MAX_IMAGE_HEIGHT << "px";
+            qCWarning(LOG_EXRPLUGIN) << "The maximum image size is limited to" << EXR_MAX_IMAGE_WIDTH << "x" << EXR_MAX_IMAGE_HEIGHT << "px";
             return false;
         }
 
@@ -596,13 +609,13 @@ bool EXRHandler::write(const QImage &image)
         pixels.resizeErase(EXR_LINES_PER_BLOCK, width);
 
         // convert the image and write into the stream
-        auto convFormat = image.hasAlphaChannel() ? QImage::Format_RGBA16FPx4 : QImage::Format_RGBX16FPx4;
+        auto convFormat = image.hasAlphaChannel() ? QImage::Format_RGBA32FPx4 : QImage::Format_RGBX32FPx4;
         ScanLineConverter slc(convFormat);
         slc.setDefaultSourceColorSpace(QColorSpace(QColorSpace::SRgb));
         slc.setTargetColorSpace(QColorSpace(QColorSpace::SRgbLinear));
         for (int y = 0, n = 0; y < height; y += n) {
             for (n = 0; n < std::min(EXR_LINES_PER_BLOCK, height - y); ++n) {
-                auto scanLine = reinterpret_cast<const qfloat16 *>(slc.convertedScanLine(image, y + n));
+                auto scanLine = reinterpret_cast<const float *>(slc.convertedScanLine(image, y + n));
                 if (scanLine == nullptr) {
                     return false;
                 }
@@ -770,7 +783,7 @@ int EXRHandler::currentImageNumber() const
 bool EXRHandler::canRead(QIODevice *device)
 {
     if (!device) {
-        qWarning("EXRHandler::canRead() called with no device");
+        qCWarning(LOG_EXRPLUGIN) << "EXRHandler::canRead() called with no device";
         return false;
     }
 
